function netinit= KSinit(y,p,W,No)
% Model parameter for the Random Walk State space model
% set A to I, initial mean and cov to zero and identity
% Find sigma_v with ARYule
% Find Q using an overapping window
% W window size, No overlap
% Written by Emtiyaz @ IISc.

N=size(y,1);
T=size(y,2);
%total number of windows
Wnum=(T-No)/(W-No);
x=zeros(Wnum,p);
e=zeros(Wnum,1);

%calculate the AR coefficients for each window
Qsum=zeros(p);
Rsum=0;
x0sum=zeros(p,1);
P0sum=zeros(p);
for i=1:N;
	Qn=zeros(p);
	for n=1:Wnum
		T1=(n-1)*(W-No)+1;
		T2=(n-1)*(W-No)+W;
		tt1=y(i,T1:T2);
		t1=squeeze(tt1);
		[t3 e(n)]=aryule(t1,p);
		x(n,:)=t3(2:p+1);
		%Qsum for estimation of Q.
		if n~=1
			xdiff=x(n,:)-x(n-1,:);
			Qn=Qn+xdiff'*xdiff;
		end
		clear t1 t2 t3 xdiff ;
	end
	Qsum=Qsum+Qn/(Wnum-1);
	Rsum=Rsum+mean(e);
	x0sum=x0sum+mean(x)';
	P0sum=P0sum+cov(x);

end
%Least Square estimate of the parameters
A=eye(p);
netinit.A=A;
netinit.Q=Qsum/N;
netinit.R=Rsum/N;
netinit.x0=x0sum/N;
P0=(P0sum/N);
netinit.P0=P0;

%check whether P0 is positive definite or not
if (~all(eig(netinit.P0))>0 )
disp('P0 is not positive semidefinite in ML estimate');
end;
if (~all(eig(netinit.Q))>0 )
disp('Q is not positive semidefinite in ML estimate');
end;

clear xdiff Qsum;
clear x1 X Y0 A1;
