function netinit=Kinit(Y,h,T,p,InitPar)
% This program refines the initial estimates 
% It runs a Kalman smoother with KSmodel parameters
% to get new states, and get Least square using those
% Y is data NxpXT
% h is the intializing data points
% T is total time points,
% p is order of the AR model
% Written by Emtiyaz @ IISc.


N=size(Y,1);
InitPar=KLSinit(Y,T,p);
A=eye(p);
Q=InitPar.Q;
R=InitPar.R;
x0=InitPar.x0;
P0=InitPar.P0;

%get the estimate  of x through kalman smoother
X2X1sum=zeros(p);
X1X1sum=zeros(p);
X2X2sum=zeros(p);
Rsum=0;
x0sum=zeros(p,1);
P0sum=zeros(p);

for n=1:N
	X=zeros(p,T);
	X1=zeros(p,T-1);
	X2=zeros(p,T-1);
	Y1=Y(n,:,:);
	h1=h(n,:)';
	%E STEP
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	out1=kalmansmoothKinit(A,Q,R,x0,P0,h1,Y1);
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	ht=out1.ht;
	X=squeeze(out1.x(1,:,:));
	X1=X(:,1:end-1);
	X2=X(:,2:end);
	if p==1
		ydiff=squeeze(Y1)-diag(ht'*X');
	else
		ydiff=squeeze(Y1)-diag(ht'*X);
	end
	Rsum=Rsum+ydiff'*ydiff;
	X2X1sum=X2X1sum+X2*X1';
	X1X1sum=X1X1sum+X1*X1';
	X2X2sum=X2X2sum+X2*X2';
	x0sum=x0sum+X(:,1);
	P0sum=P0sum+cov(X');
	clear out1 X X1 X2 ydiff ht;
end
%Least Square estimate of the parameters
A=X2X1sum*inv(X1X1sum);
Q=(X2X2sum-A*X2X1sum')/(N*(T-1));
R=Rsum/(N*T);
x0=x0sum/N;
P0=P0sum/N;

netinit.A=A;
netinit.Q=Q;
netinit.R=R;
netinit.x0=x0;
netinit.P0=P0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%[
function  out=kalmansmoothKinit(A,Q,R,x0,P0,h1,Y);
% Kalman Smoother

out=struct('x',[],'var0',[],'Ptsum',[],'ht',[]);
[N p T]=size(Y);
K=length(x0);
tiny=exp(-700);
I=eye(K);
const=(2*pi)^(-p/2);
problem=0;
lik=0;

Xpre=zeros(N,K);   % P(x_t | y_1 ... y_{t-1})
Xcur=zeros(N,K,T);   % P(x_t | y_1 ... y_t)
Xfin=zeros(N,K,T);   % P(x_t | y_1 ... y_T)    given all outputs

Ppre=zeros(K,K,T);
Pcur=zeros(K,K,T);
Pfin=zeros(K,K,T); 

Pt=zeros(K,K); 
Pcov=zeros(K,K); 
Kcur=zeros(K,p);
invP=zeros(p,p);
J=zeros(K,K,T);

%%%%%%%%%%%%%%%
% FORWARD PASS

R=R+(R==0)*tiny;

Xpre=ones(N,1)*x0';
Ppre(:,:,1)=P0;
invR=diag(1./R);

ht=h1;
htemp2=zeros(K,T+1);
htemp2(:,1)=h1;

for t=1:T
  if (K<p)
    temp1=rdiv(C,R);
    temp2=temp1*Ppre(:,:,t); % inv(R)*C*Ppre
    temp3=C'*temp2;
    temp4=inv(I+temp3)*temp1';
    invP=invR-temp2*temp4; 
    CP= temp1' - temp3*temp4;    % C'*invP
  else
    temp1=diag(R)+ht'*Ppre(:,:,t)*ht;
    invP=inv(temp1);
    CP=ht*invP;
  end;
  Kcur=Ppre(:,:,t)*CP;
  KC=Kcur*ht';
  Ydiff=Y(:,:,t)-Xpre*ht;
  Xcur(:,:,t)=Xpre+Ydiff*Kcur'; 
  Pcur(:,:,t)=Ppre(:,:,t)-KC*Ppre(:,:,t);
  if (t<T)
    Xpre=Xcur(:,:,t)*A';
    Ppre(:,:,t+1)=A*Pcur(:,:,t)*A'+Q;
  end;

  % calculate likelihood
  detiP=sqrt(det(invP));
  if (isreal(detiP) & detiP>0)
    lik=lik+N*log(detiP)-0.5*sum(sum(Ydiff.*(Ydiff*invP)));
  else
    problem=1;
  end;
   %initialize ht for the next cycle
   if K~=1
	ht=[Y(:,:,t) ht(1:K-1)']';
	htemp2(:,t+1)=ht;
   else
	temp1=Y(:,:,t);
	htemp2(:,t+1)=ht;
   end

end;  

lik=lik+N*T*log(const);

%%%%%%%%%%%%%%%
% BACKWARD PASS

A1=zeros(K);
A2=zeros(K);
A3=zeros(K);
Ptsum=zeros(K);
YX=zeros(p,K);
hPh=0;

  
t=T; 
%ht=[ht(2:K)' Y(:,:,T-K)]';
ht=htemp2(:,t);

Xfin(:,:,t)=Xcur(:,:,t);
Pfin(:,:,t)=Pcur(:,:,t); 
Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
A2= -Pt;
Ptsum=Pt;

YX=(Y(:,:,t)'*Xfin(:,:,t))*ht;
hPh=ht'*Pt*ht;

for t=(T-1):-1:1

  %initialize th for this cycle
  ht=htemp2(:,t);

  J(:,:,t)=Pcur(:,:,t)*A'*inv(Ppre(:,:,t+1));
  Xfin(:,:,t)=Xcur(:,:,t)+(Xfin(:,:,t+1)-Xcur(:,:,t)*A')*J(:,:,t)';
  Pfin(:,:,t)=Pcur(:,:,t)+J(:,:,t)*(Pfin(:,:,t+1)-Ppre(:,:,t+1))*J(:,:,t)';
  Pt=Pfin(:,:,t) + Xfin(:,:,t)'*Xfin(:,:,t)/N; 
  Ptsum=Ptsum+Pt;
  YX=YX+(Y(:,:,t)'*Xfin(:,:,t))*ht;
  hPh=hPh+ht'*Pt*ht;
end;

A3= Ptsum-Pt;
A2= Ptsum+A2;

t=T;  
Pcov=(I-KC)*A*Pcur(:,:,t-1);
A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;

for t=(T-1):-1:2
  Pcov=(Pcur(:,:,t)+J(:,:,t)*(Pcov-A*Pcur(:,:,t)))*J(:,:,t-1)';
  A1=A1+Pcov+Xfin(:,:,t)'*Xfin(:,:,t-1)/N;
end;    


if problem 
  fprintf(' problem  '); problem=0;
end;

out.x=Xfin;
out.var0=Pfin(:,:,1);
out.Ptsum=Ptsum;
out.ht=htemp2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function netinit=KLSinit(y,T,p)

N=size(y,1);
fs=128;
W=32;
No=24;

%total number of windows
Wnum=floor((T-No)/(W-No));
x=zeros(Wnum,p);
e=zeros(Wnum,1);

%calculate the AR coefficients for each window
X2X1sum=zeros(p);
X1X1sum=zeros(p);
X2X2sum=zeros(p);
Rsum=0;
x0sum=zeros(p,1);
P0sum=zeros(p);
for i=1:N;
	Qn=zeros(p);
	for n=1:Wnum
		T1=(n-1)*(W-No)+1;
		T2=(n-1)*(W-No)+W;
		tt1=y(i,1,T1:T2);
		t1=squeeze(tt1);
		[t3 e(n)]=aryule(t1,p);
		x(n,:)=t3(2:p+1);
		%Qsum for estimation of Q.
		clear t1 t2 t3 xdiff ;
	end
	X1=x(1:end-1,:)';
	X2=x(2:end,:)';
	X2X1sum=X2X1sum+X2*X1';
	X1X1sum=X1X1sum+X1*X1';
	X2X2sum=X2X2sum+X2*X2';
	Rsum=Rsum+mean(e);
	x0sum=x0sum+mean(x)';
	P0sum=P0sum+cov(x);
	clear X1 X2 x;
end
%Least Square estimate of the parameters
A=X2X1sum*inv(X1X1sum);
Q=(X2X2sum-A*X2X1sum')/(N*(Wnum-1));
R=Rsum/N;
x0=x0sum/N;
P0=P0sum/N;

netinit.A=A;
netinit.Q=Q;
netinit.R=R;
netinit.x0=x0;
netinit.P0=P0;

%check whether P0 is positive definite or not
if (~all(eig(netinit.P0))>0 )
disp('P0 is not positive semidefinite in ML estimate');
end;
if (~all(eig(netinit.Q))>0 )
disp('Q is not positive semidefinite in ML estimate');
end;


