% Example file for IF estimation
% Illustrates the use of EMKS for Instantaneous frequency (IF)
% estimation and compares with RLS and KS 
% please see the papers for details
% Written by Emtiyaz @ IISc.

% Generate Data for Linear Chirp
disp('generating data...');
[ID,D]=data(3,1); %option 3 is for linear chirp
[IDtrain,Dtrain]=data(3,30); %option 3 is for linear chirp
N=ID.N;
%%%%%%%%%%%%%%%%%%%%%%
% rls algorithm for IFestimation
disp('RLS algorithm...');
p=4; a0=zeros(p,1); P0=eye(p);
xrls = rls(D.y,a0,P0,.85); % state estimation
% IF estimation
frls=peakpick(squeeze(xrls(1,:,:))); 
%%%%%%%%%%%%%%%%%%%%%%
% KS algorithm
disp('Kalman smoother algorithm without EM ...');
p=4; 
KSmodel = KSinit(Dtrain.y,p,ID.T/8,ID.T/16); %Model 
xks = KalmanSmooth(D.y,KSmodel,p); % state estimation
% IF estimation
fks=peakpick(squeeze(xks(1,:,:)));
%%%%%%%%%%%%%%%%%%%%%%
% EMKS algorithm
disp('EM algorithm...');
p=4;
% EM algorithm to tvar estimation
EMKSmodel = emtvar(Dtrain.y,p,KSmodel,1000,0.01);
xemks = KalmanSmooth(D.y,EMKSmodel,p);
% IF estimation
femks=peakpick(squeeze(xemks(1,:,:)));

%plot
figure
plot(D.ft(4:end),'b');
hold on;
plot(frls,'r');
plot(fks,'g');
plot(femks,'k','LineWidth',2);
axis([1 256 0 20]);
legend('Actual','RLS','KS','EMKS');

clear frls femks fks;
% Average Analysis 
N = 100;
[ID,D]=data(3,100); %option 3 is for linear chirp
xrls = rls(D.y,a0,P0,.898); % state estimation
for i =1:N
  frls(i,:)=peakpick(squeeze(xrls(i,:,:))); 
end
xks = KalmanSmooth(D.y,KSmodel,p); % state estimation
for i=1:N
  fks(i,:)=peakpick(squeeze(xks(i,:,:)));
end
xemks = KalmanSmooth(D.y,EMKSmodel,p);
for i=1:N
  femks(i,:)=peakpick(squeeze(xemks(i,:,:)));
end
%plot
figure
subplot(311)
m = squeeze(mean(frls,1));
v =squeeze(std(frls,1));
plot(D.ft);
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
title('RLS')
axis([1 256 0 20]);

subplot(312)
m = squeeze(mean(fks,1));
v =squeeze(std(fks,1));
plot(D.ft);
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
title('KS')
axis([1 256 0 20]);

subplot(313)
m = squeeze(mean(femks,1));
v =squeeze(std(femks,1));
plot(D.ft);
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
title('EMKS')
axis([1 256 0 20]);



