% Example file for TVAR(2) simulation
% Illustrates the use of EMKS for AR parameter estimation
% and compares with RLS and KS 
% please see the papers for details
% Written by Emtiyaz @ IISc.

% Generate Data for Linear Chirp
disp('generating data...');
[ID,D]=data(2,1); %option 3 is for linear chirp
[IDtrain,Dtrain]=data(2,100); %option 3 is for linear chirp
N=ID.N;
figure
zplot(D.z)
%%%%%%%%%%%%%%%%%%%%%%
% rls algorithm for IFestimation
disp('RLS algorithm...');
p=2; a0=zeros(p,1); P0=eye(p);
xrls = rls(D.y,a0,P0,.898); % state estimation
%%%%%%%%%%%%%%%%%%%%%%
% KS algorithm
disp('Kalman smoother algorithm without EM ...');
p=2; 
KSmodel = KSinit(Dtrain.y,p,ID.T/8,ID.T/16); %Model 
xks = KalmanSmooth(D.y,KSmodel,p); % state estimation
%%%%%%%%%%%%%%%%%%%%%%
% EMKS algorithm
disp('EM algorithm...');
p=2;
% EM algorithm to tvar estimation
EMKSmodel = emtvar(Dtrain.y,p,KSmodel,100,0.01);
xemks = KalmanSmooth(D.y,EMKSmodel,p);
%%%%%%%%%%%%%%%%%%%%%%
% EMKS algorithm
%plot
figure
subplot(211)
plot(D.a(1,p:end),'b');
hold on;
plot(squeeze(xrls(1,1,:)),'r');
plot(squeeze(xks(1,1,:)),'g');
plot(squeeze(xemks(1,1,:)),'k','linewidth',2);
axis([1 256 0.5 2]);
legend('Actual','RLS','KS','EMKS');
subplot(212)
plot(D.a(2,p:end),'b');
hold on;
plot(squeeze(xrls(1,2,:)),'r');
plot(squeeze(xks(1,2,:)),'g');
plot(squeeze(xemks(1,2,:)),'k','linewidth',2);
axis([1 256 -1.2 -0.2]);


% Average Analysis 
[ID,D]=data(2,100); %option 3 is for linear chirp
xrls = rls(D.y,a0,P0,.898); % state estimation
xks = KalmanSmooth(D.y,KSmodel,p); % state estimation
xemks = KalmanSmooth(D.y,EMKSmodel,p);
%plot
figure
subplot(231)
m = squeeze(mean(xrls(:,1,:),1));
v =squeeze( std(xrls(:,1,:),1));
plot(D.a(1,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 0.5 2]);
title('RLS')
subplot(234)
m = squeeze(mean(xrls(:,2,:),1));
v =squeeze( std(xrls(:,2,:),1));
plot(D.a(2,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 -1.2 -0.2]);

subplot(232)
m =squeeze( mean(xks(:,1,:),1));
v =squeeze( std(xks(:,1,:),1));
plot(D.a(1,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 0.5 2]);
title('KS')
subplot(235)
m =squeeze( mean(xks(:,2,:),1));
v =squeeze( std(xks(:,2,:),1));
plot(D.a(2,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 -1.2 -0.2]);

subplot(233)
m =squeeze( mean(xemks(:,1,:),1));
v =squeeze( std(xemks(:,1,:),1));
plot(D.a(1,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 0.5 2]);
title('EMKS')
subplot(236)
m =squeeze( mean(xemks(:,2,:),1));
v =squeeze( std(xemks(:,2,:),1));
plot(D.a(2,p:end));
hold on;
plot(m,'linewidth',2);
plot(m+v);
plot(m-v);
axis([1 256 -1.2 -0.2]);





